@@ -1,5 +1,5 @@
 /*
- * Copyright (C)2011 D. R. Commander.  All Rights Reserved.
+ * Copyright (C)2011-2013 D. R. Commander.  All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -36,7 +36,7 @@ import java.nio.*;
  */
 public class TJDecompressor {
 
-  private final static String NO_ASSOC_ERROR =
+  private static final String NO_ASSOC_ERROR =
     "No JPEG image is associated with this instance";
 
   /**
@@ -82,7 +82,7 @@ public class TJDecompressor {
    * @param imageSize size of the JPEG image (in bytes)
    */
   public void setJPEGImage(byte[] jpegImage, int imageSize) throws Exception {
-    if(jpegImage == null || imageSize < 1)
+    if (jpegImage == null || imageSize < 1)
       throw new Exception("Invalid argument in setJPEGImage()");
     jpegBuf = jpegImage;
     jpegBufSize = imageSize;
@@ -97,7 +97,8 @@ public class TJDecompressor {
    * instance
    */
   public int getWidth() throws Exception {
-    if(jpegWidth < 1) throw new Exception(NO_ASSOC_ERROR);
+    if (jpegWidth < 1)
+      throw new Exception(NO_ASSOC_ERROR);
     return jpegWidth;
   }
 
@@ -109,7 +110,8 @@ public class TJDecompressor {
    * instance
    */
   public int getHeight() throws Exception {
-    if(jpegHeight < 1) throw new Exception(NO_ASSOC_ERROR);
+    if (jpegHeight < 1)
+      throw new Exception(NO_ASSOC_ERROR);
     return jpegHeight;
   }
 
@@ -121,8 +123,9 @@ public class TJDecompressor {
    * associated with this decompressor instance
    */
   public int getSubsamp() throws Exception {
-    if(jpegSubsamp < 0) throw new Exception(NO_ASSOC_ERROR);
-    if(jpegSubsamp >= TJ.NUMSAMP)
+    if (jpegSubsamp < 0)
+      throw new Exception(NO_ASSOC_ERROR);
+    if (jpegSubsamp >= TJ.NUMSAMP)
       throw new Exception("JPEG header information is invalid");
     return jpegSubsamp;
   }
@@ -133,7 +136,8 @@ public class TJDecompressor {
    * @return the JPEG image buffer associated with this decompressor instance
    */
   public byte[] getJPEGBuf() throws Exception {
-    if(jpegBuf == null) throw new Exception(NO_ASSOC_ERROR);
+    if (jpegBuf == null)
+      throw new Exception(NO_ASSOC_ERROR);
     return jpegBuf;
   }
 
@@ -145,13 +149,14 @@ public class TJDecompressor {
    * decompressor instance
    */
   public int getJPEGSize() throws Exception {
-    if(jpegBufSize < 1) throw new Exception(NO_ASSOC_ERROR);
+    if (jpegBufSize < 1)
+      throw new Exception(NO_ASSOC_ERROR);
     return jpegBufSize;
   }
 
 
   /**
-   * Returns the width of the largest scaled down image that the TurboJPEG
+   * Returns the width of the largest scaled-down image that the TurboJPEG
    * decompressor can generate without exceeding the desired image width and
    * height.
    *
@@ -165,33 +170,35 @@ public class TJDecompressor {
    * image (in other words, the height will not be considered when determining
    * the scaled image size.)
    *
-   * @return the width of the largest scaled down image that the TurboJPEG
+   * @return the width of the largest scaled-down image that the TurboJPEG
    * decompressor can generate without exceeding the desired image width and
    * height
    */
   public int getScaledWidth(int desiredWidth, int desiredHeight)
-    throws Exception {
-    if(jpegWidth < 1 || jpegHeight < 1)
+                            throws Exception {
+    if (jpegWidth < 1 || jpegHeight < 1)
       throw new Exception(NO_ASSOC_ERROR);
-    if(desiredWidth < 0 || desiredHeight < 0)
+    if (desiredWidth < 0 || desiredHeight < 0)
       throw new Exception("Invalid argument in getScaledWidth()");
-    TJScalingFactor sf[] = TJ.getScalingFactors();
-    if(desiredWidth == 0) desiredWidth = jpegWidth;
-    if(desiredHeight == 0) desiredHeight = jpegHeight;
+    TJScalingFactor[] sf = TJ.getScalingFactors();
+    if (desiredWidth == 0)
+      desiredWidth = jpegWidth;
+    if (desiredHeight == 0)
+      desiredHeight = jpegHeight;
     int scaledWidth = jpegWidth, scaledHeight = jpegHeight;
-    for(int i = 0; i < sf.length; i++) {
+    for (int i = 0; i < sf.length; i++) {
       scaledWidth = sf[i].getScaled(jpegWidth);
       scaledHeight = sf[i].getScaled(jpegHeight);
-      if(scaledWidth <= desiredWidth && scaledHeight <= desiredHeight)
+      if (scaledWidth <= desiredWidth && scaledHeight <= desiredHeight)
         break;
     }
-    if(scaledWidth > desiredWidth || scaledHeight > desiredHeight)
+    if (scaledWidth > desiredWidth || scaledHeight > desiredHeight)
       throw new Exception("Could not scale down to desired image dimensions");
     return scaledWidth;
   }
 
   /**
-   * Returns the height of the largest scaled down image that the TurboJPEG
+   * Returns the height of the largest scaled-down image that the TurboJPEG
    * decompressor can generate without exceeding the desired image width and
    * height.
    *
@@ -205,27 +212,29 @@ public class TJDecompressor {
    * image (in other words, the height will not be considered when determining
    * the scaled image size.)
    *
-   * @return the height of the largest scaled down image that the TurboJPEG
+   * @return the height of the largest scaled-down image that the TurboJPEG
    * decompressor can generate without exceeding the desired image width and
    * height
    */
   public int getScaledHeight(int desiredWidth, int desiredHeight)
-    throws Exception {
-    if(jpegWidth < 1 || jpegHeight < 1)
+                             throws Exception {
+    if (jpegWidth < 1 || jpegHeight < 1)
       throw new Exception(NO_ASSOC_ERROR);
-    if(desiredWidth < 0 || desiredHeight < 0)
+    if (desiredWidth < 0 || desiredHeight < 0)
       throw new Exception("Invalid argument in getScaledHeight()");
-    TJScalingFactor sf[] = TJ.getScalingFactors();
-    if(desiredWidth == 0) desiredWidth = jpegWidth;
-    if(desiredHeight == 0) desiredHeight = jpegHeight;
+    TJScalingFactor[] sf = TJ.getScalingFactors();
+    if (desiredWidth == 0)
+      desiredWidth = jpegWidth;
+    if (desiredHeight == 0)
+      desiredHeight = jpegHeight;
     int scaledWidth = jpegWidth, scaledHeight = jpegHeight;
-    for(int i = 0; i < sf.length; i++) {
+    for (int i = 0; i < sf.length; i++) {
       scaledWidth = sf[i].getScaled(jpegWidth);
       scaledHeight = sf[i].getScaled(jpegHeight);
-      if(scaledWidth <= desiredWidth && scaledHeight <= desiredHeight)
+      if (scaledWidth <= desiredWidth && scaledHeight <= desiredHeight)
         break;
     }
-    if(scaledWidth > desiredWidth || scaledHeight > desiredHeight)
+    if (scaledWidth > desiredWidth || scaledHeight > desiredHeight)
       throw new Exception("Could not scale down to desired image dimensions");
     return scaledHeight;
   }
@@ -239,47 +248,76 @@ public class TJDecompressor {
    * where <code>scaledHeight</code> can be determined by calling <code>
    * scalingFactor.{@link TJScalingFactor#getScaled getScaled}(jpegHeight)
    * </code> with one of the scaling factors returned from {@link
-   * TJ#getScalingFactors} or by calling {@link #getScaledHeight}.
+   * TJ#getScalingFactors} or by calling {@link #getScaledHeight}.  However,
+   * the buffer may also be larger than the dimensions of the JPEG image, in
+   * which case the <code>x</code>, <code>y</code>, and <code>pitch</code>
+   * parameters can be used to specify the region into which the JPEG image
+   * should be decompressed.
    *
-   * @param desiredWidth desired width (in pixels) of the decompressed image.
-   * If the desired image dimensions are smaller than the dimensions of the
-   * JPEG image being decompressed, then TurboJPEG will use scaling in the JPEG
-   * decompressor to generate the largest possible image that will fit within
-   * the desired dimensions.  Setting this to 0 is the same as setting it to
-   * the width of the JPEG image (in other words, the width will not be
-   * considered when determining the scaled image size.)
+   * @param x x offset (in pixels) of the region into which the JPEG image
+   * should be decompressed, relative to the start of <code>dstBuf</code>.
+   *
+   * @param y y offset (in pixels) of the region into which the JPEG image
+   * should be decompressed, relative to the start of <code>dstBuf</code>.
+   *
+   * @param desiredWidth desired width (in pixels) of the decompressed image
+   * (or image region.)  If the desired image dimensions are different than the
+   * dimensions of the JPEG image being decompressed, then TurboJPEG will use
+   * scaling in the JPEG decompressor to generate the largest possible image
+   * that will fit within the desired dimensions.  Setting this to 0 is the
+   * same as setting it to the width of the JPEG image (in other words, the
+   * width will not be considered when determining the scaled image size.)
    *
    * @param pitch bytes per line of the destination image.  Normally, this
    * should be set to <code>scaledWidth * TJ.pixelSize(pixelFormat)</code> if
    * the decompressed image is unpadded, but you can use this to, for instance,
-   * pad each line of the decompressed image to a 4-byte boundary.  NOTE:
+   * pad each line of the decompressed image to a 4-byte boundary or to
+   * decompress the JPEG image into a region of a larger image.  NOTE:
    * <code>scaledWidth</code> can be determined by calling <code>
    * scalingFactor.{@link TJScalingFactor#getScaled getScaled}(jpegWidth)
    * </code> or by calling {@link #getScaledWidth}.  Setting this parameter to
    * 0 is the equivalent of setting it to <code>scaledWidth *
    * TJ.pixelSize(pixelFormat)</code>.
    *
-   * @param desiredHeight desired height (in pixels) of the decompressed image.
-   * If the desired image dimensions are smaller than the dimensions of the
-   * JPEG image being decompressed, then TurboJPEG will use scaling in the JPEG
-   * decompressor to generate the largest possible image that will fit within
-   * the desired dimensions.  Setting this to 0 is the same as setting it to
-   * the height of the JPEG image (in other words, the height will not be
-   * considered when determining the scaled image size.)
+   * @param desiredHeight desired height (in pixels) of the decompressed image
+   * (or image region.)  If the desired image dimensions are different than the
+   * dimensions of the JPEG image being decompressed, then TurboJPEG will use
+   * scaling in the JPEG decompressor to generate the largest possible image
+   * that will fit within the desired dimensions.  Setting this to 0 is the
+   * same as setting it to the height of the JPEG image (in other words, the
+   * height will not be considered when determining the scaled image size.)
    *
    * @param pixelFormat pixel format of the decompressed image (one of
    * {@link TJ TJ.PF_*})
    *
    * @param flags the bitwise OR of one or more of {@link TJ TJ.FLAG_*}
    */
-  public void decompress(byte[] dstBuf, int desiredWidth, int pitch,
-    int desiredHeight, int pixelFormat, int flags) throws Exception {
-    if(jpegBuf == null) throw new Exception(NO_ASSOC_ERROR);
-    if(dstBuf == null || desiredWidth < 0 || pitch < 0 || desiredHeight < 0
-      || pixelFormat < 0 || pixelFormat >= TJ.NUMPF || flags < 0)
+  public void decompress(byte[] dstBuf, int x, int y, int desiredWidth,
+                         int pitch, int desiredHeight, int pixelFormat,
+                         int flags) throws Exception {
+    if (jpegBuf == null)
+      throw new Exception(NO_ASSOC_ERROR);
+    if (dstBuf == null || x < 0 || y < 0 || desiredWidth < 0 || pitch < 0 ||
+        desiredHeight < 0 || pixelFormat < 0 || pixelFormat >= TJ.NUMPF ||
+        flags < 0)
       throw new Exception("Invalid argument in decompress()");
-    decompress(jpegBuf, jpegBufSize, dstBuf, desiredWidth, pitch,
-      desiredHeight, pixelFormat, flags);
+    if (x > 0 || y > 0)
+      decompress(jpegBuf, jpegBufSize, dstBuf, x, y, desiredWidth, pitch,
+                 desiredHeight, pixelFormat, flags);
+    else
+      decompress(jpegBuf, jpegBufSize, dstBuf, desiredWidth, pitch,
+                 desiredHeight, pixelFormat, flags);
+  }
+
+  /**
+   * @deprecated Use
+   * {@link #decompress(byte[], int, int, int, int, int, int, int)} instead.
+   */
+  public void decompress(byte[] dstBuf, int desiredWidth, int pitch,
+                         int desiredHeight, int pixelFormat, int flags)
+                         throws Exception {
+    decompress(dstBuf, 0, 0, desiredWidth, pitch, desiredHeight, pixelFormat,
+               flags);
   }
 
   /**
@@ -287,13 +325,16 @@ public class TJDecompressor {
    * instance and return a buffer containing the decompressed image.
    *
    * @param desiredWidth see
-   * {@link #decompress(byte[], int, int, int, int, int)} for description
+   * {@link #decompress(byte[], int, int, int, int, int, int, int)}
+   * for description
    *
    * @param pitch see
-   * {@link #decompress(byte[], int, int, int, int, int)} for description
+   * {@link #decompress(byte[], int, int, int, int, int, int, int)}
+   * for description
    *
    * @param desiredHeight see
-   * {@link #decompress(byte[], int, int, int, int, int)} for description
+   * {@link #decompress(byte[], int, int, int, int, int, int, int)}
+   * for description
    *
    * @param pixelFormat pixel format of the decompressed image (one of
    * {@link TJ TJ.PF_*})
@@ -303,14 +344,15 @@ public class TJDecompressor {
    * @return a buffer containing the decompressed image
    */
   public byte[] decompress(int desiredWidth, int pitch, int desiredHeight,
-    int pixelFormat, int flags) throws Exception {
-    if(desiredWidth < 0 || pitch < 0 || desiredHeight < 0
-      || pixelFormat < 0 || pixelFormat >= TJ.NUMPF || flags < 0)
+                           int pixelFormat, int flags) throws Exception {
+    if (desiredWidth < 0 || pitch < 0 || desiredHeight < 0 ||
+        pixelFormat < 0 || pixelFormat >= TJ.NUMPF || flags < 0)
       throw new Exception("Invalid argument in decompress()");
     int pixelSize = TJ.getPixelSize(pixelFormat);
     int scaledWidth = getScaledWidth(desiredWidth, desiredHeight);
     int scaledHeight = getScaledHeight(desiredWidth, desiredHeight);
-    if(pitch == 0) pitch = scaledWidth * pixelSize;
+    if (pitch == 0)
+      pitch = scaledWidth * pixelSize;
     byte[] buf = new byte[pitch * scaledHeight];
     decompress(buf, desiredWidth, pitch, desiredHeight, pixelFormat, flags);
     return buf;
@@ -321,11 +363,11 @@ public class TJDecompressor {
    * instance and output a YUV planar image to the given destination buffer.
    * This method performs JPEG decompression but leaves out the color
    * conversion step, so a planar YUV image is generated instead of an RGB
-   * image.  The padding of the planes in this image is the same as the images
-   * generated by {@link TJCompressor#encodeYUV(byte[], int)}.  Note that, if
-   * the width or height of the image is not an even multiple of the MCU block
-   * size (see {@link TJ#getMCUWidth} and {@link TJ#getMCUHeight}), then an
-   * intermediate buffer copy will be performed within TurboJPEG.
+   * image.  The padding of the planes in this image is the same as in the
+   * images generated by {@link TJCompressor#encodeYUV(byte[], int)}.  Note
+   * that, if the width or height of the image is not an even multiple of the
+   * MCU block size (see {@link TJ#getMCUWidth} and {@link TJ#getMCUHeight}),
+   * then an intermediate buffer copy will be performed within TurboJPEG.
    *
    * @param dstBuf buffer that will receive the YUV planar image.  Use
    * {@link TJ#bufSizeYUV} to determine the appropriate size for this buffer
@@ -334,13 +376,14 @@ public class TJDecompressor {
    * @param flags the bitwise OR of one or more of {@link TJ TJ.FLAG_*}
    */
   public void decompressToYUV(byte[] dstBuf, int flags) throws Exception {
-    if(jpegBuf == null) throw new Exception(NO_ASSOC_ERROR);
-    if(dstBuf == null || flags < 0)
+    if (jpegBuf == null)
+      throw new Exception(NO_ASSOC_ERROR);
+    if (dstBuf == null || flags < 0)
       throw new Exception("Invalid argument in decompressToYUV()");
     decompressToYUV(jpegBuf, jpegBufSize, dstBuf, flags);
   }
 
-  
+
   /**
    * Decompress the JPEG source image associated with this decompressor
    * instance and return a buffer containing a YUV planar image.  See {@link
@@ -351,17 +394,80 @@ public class TJDecompressor {
    * @return a buffer containing a YUV planar image
    */
   public byte[] decompressToYUV(int flags) throws Exception {
-    if(flags < 0)
+    if (flags < 0)
       throw new Exception("Invalid argument in decompressToYUV()");
-    if(jpegWidth < 1 || jpegHeight < 1 || jpegSubsamp < 0)
+    if (jpegWidth < 1 || jpegHeight < 1 || jpegSubsamp < 0)
       throw new Exception(NO_ASSOC_ERROR);
-    if(jpegSubsamp >= TJ.NUMSAMP)
+    if (jpegSubsamp >= TJ.NUMSAMP)
       throw new Exception("JPEG header information is invalid");
     byte[] buf = new byte[TJ.bufSizeYUV(jpegWidth, jpegHeight, jpegSubsamp)];
     decompressToYUV(buf, flags);
     return buf;
   }
 
+  /**
+   * Decompress the JPEG source image associated with this decompressor
+   * instance and output a decompressed image to the given destination buffer.
+   *
+   * @param dstBuf buffer that will receive the decompressed image.  This
+   * buffer should normally be <code>stride * scaledHeight</code> pixels in
+   * size, where <code>scaledHeight</code> can be determined by calling <code>
+   * scalingFactor.{@link TJScalingFactor#getScaled getScaled}(jpegHeight)
+   * </code> with one of the scaling factors returned from {@link
+   * TJ#getScalingFactors} or by calling {@link #getScaledHeight}.  However,
+   * the buffer may also be larger than the dimensions of the JPEG image, in
+   * which case the <code>x</code>, <code>y</code>, and <code>stride</code>
+   * parameters can be used to specify the region into which the JPEG image
+   * should be decompressed.
+   *
+   * @param x x offset (in pixels) of the region into which the JPEG image
+   * should be decompressed, relative to the start of <code>dstBuf</code>.
+   *
+   * @param y y offset (in pixels) of the region into which the JPEG image
+   * should be decompressed, relative to the start of <code>dstBuf</code>.
+   *
+   * @param desiredWidth desired width (in pixels) of the decompressed image
+   * (or image region.)  If the desired image dimensions are different than the
+   * dimensions of the JPEG image being decompressed, then TurboJPEG will use
+   * scaling in the JPEG decompressor to generate the largest possible image
+   * that will fit within the desired dimensions.  Setting this to 0 is the
+   * same as setting it to the width of the JPEG image (in other words, the
+   * width will not be considered when determining the scaled image size.)
+   *
+   * @param stride pixels per line of the destination image.  Normally, this
+   * should be set to <code>scaledWidth</code>, but you can use this to, for
+   * instance, decompress the JPEG image into a region of a larger image.
+   * NOTE: <code>scaledWidth</code> can be determined by calling <code>
+   * scalingFactor.{@link TJScalingFactor#getScaled getScaled}(jpegWidth)
+   * </code> or by calling {@link #getScaledWidth}.  Setting this parameter to
+   * 0 is the equivalent of setting it to <code>scaledWidth</code>.
+   *
+   * @param desiredHeight desired height (in pixels) of the decompressed image
+   * (or image region.)  If the desired image dimensions are different than the
+   * dimensions of the JPEG image being decompressed, then TurboJPEG will use
+   * scaling in the JPEG decompressor to generate the largest possible image
+   * that will fit within the desired dimensions.  Setting this to 0 is the
+   * same as setting it to the height of the JPEG image (in other words, the
+   * height will not be considered when determining the scaled image size.)
+   *
+   * @param pixelFormat pixel format of the decompressed image (one of
+   * {@link TJ TJ.PF_*})
+   *
+   * @param flags the bitwise OR of one or more of {@link TJ TJ.FLAG_*}
+   */
+  public void decompress(int[] dstBuf, int x, int y, int desiredWidth,
+                         int stride, int desiredHeight, int pixelFormat,
+                         int flags) throws Exception {
+    if (jpegBuf == null)
+      throw new Exception(NO_ASSOC_ERROR);
+    if (dstBuf == null || x < 0 || y < 0 || desiredWidth < 0 || stride < 0 ||
+        desiredHeight < 0 || pixelFormat < 0 || pixelFormat >= TJ.NUMPF ||
+        flags < 0)
+      throw new Exception("Invalid argument in decompress()");
+    decompress(jpegBuf, jpegBufSize, dstBuf, x, y, desiredWidth, stride,
+               desiredHeight, pixelFormat, flags);
+  }
+
   /**
    * Decompress the JPEG source image associated with this decompressor
    * instance and output a decompressed image to the given
@@ -373,16 +479,16 @@ public class TJDecompressor {
    * @param flags the bitwise OR of one or more of {@link TJ TJ.FLAG_*}
    */
   public void decompress(BufferedImage dstImage, int flags) throws Exception {
-    if(dstImage == null || flags < 0)
+    if (dstImage == null || flags < 0)
       throw new Exception("Invalid argument in decompress()");
     int desiredWidth = dstImage.getWidth();
     int desiredHeight = dstImage.getHeight();
     int scaledWidth = getScaledWidth(desiredWidth, desiredHeight);
     int scaledHeight = getScaledHeight(desiredWidth, desiredHeight);
-    if(scaledWidth != desiredWidth || scaledHeight != desiredHeight)
+    if (scaledWidth != desiredWidth || scaledHeight != desiredHeight)
       throw new Exception("BufferedImage dimensions do not match a scaled image size that TurboJPEG is capable of generating.");
     int pixelFormat;  boolean intPixels = false;
-    if(byteOrder == null)
+    if (byteOrder == null)
       byteOrder = ByteOrder.nativeOrder();
     switch(dstImage.getType()) {
       case BufferedImage.TYPE_3BYTE_BGR:
@@ -393,20 +499,20 @@ public class TJDecompressor {
       case BufferedImage.TYPE_BYTE_GRAY:
         pixelFormat = TJ.PF_GRAY;  break;
       case BufferedImage.TYPE_INT_BGR:
-        if(byteOrder == ByteOrder.BIG_ENDIAN)
+        if (byteOrder == ByteOrder.BIG_ENDIAN)
           pixelFormat = TJ.PF_XBGR;
         else
           pixelFormat = TJ.PF_RGBX;
         intPixels = true;  break;
       case BufferedImage.TYPE_INT_RGB:
-        if(byteOrder == ByteOrder.BIG_ENDIAN)
+        if (byteOrder == ByteOrder.BIG_ENDIAN)
           pixelFormat = TJ.PF_XRGB;
         else
           pixelFormat = TJ.PF_BGRX;
         intPixels = true;  break;
       case BufferedImage.TYPE_INT_ARGB:
       case BufferedImage.TYPE_INT_ARGB_PRE:
-        if(byteOrder == ByteOrder.BIG_ENDIAN)
+        if (byteOrder == ByteOrder.BIG_ENDIAN)
           pixelFormat = TJ.PF_ARGB;
         else
           pixelFormat = TJ.PF_BGRA;
@@ -415,21 +521,21 @@ public class TJDecompressor {
         throw new Exception("Unsupported BufferedImage format");
     }
     WritableRaster wr = dstImage.getRaster();
-    if(intPixels) {
+    if (intPixels) {
       SinglePixelPackedSampleModel sm =
         (SinglePixelPackedSampleModel)dstImage.getSampleModel();
-      int pitch = sm.getScanlineStride();
+      int stride = sm.getScanlineStride();
       DataBufferInt db = (DataBufferInt)wr.getDataBuffer();
       int[] buf = db.getData();
-      if(jpegBuf == null) throw new Exception(NO_ASSOC_ERROR);
-      decompress(jpegBuf, jpegBufSize, buf, scaledWidth, pitch, scaledHeight,
-        pixelFormat, flags);
-    }
-    else {
+      if (jpegBuf == null)
+        throw new Exception(NO_ASSOC_ERROR);
+      decompress(jpegBuf, jpegBufSize, buf, scaledWidth, stride, scaledHeight,
+                 pixelFormat, flags);
+    } else {
       ComponentSampleModel sm =
         (ComponentSampleModel)dstImage.getSampleModel();
       int pixelSize = sm.getPixelStride();
-      if(pixelSize != TJ.getPixelSize(pixelFormat))
+      if (pixelSize != TJ.getPixelSize(pixelFormat))
         throw new Exception("Inconsistency between pixel format and pixel size in BufferedImage");
       int pitch = sm.getScanlineStride();
       DataBufferByte db = (DataBufferByte)wr.getDataBuffer();
@@ -444,10 +550,12 @@ public class TJDecompressor {
    * decompressed image.
    *
    * @param desiredWidth see
-   * {@link #decompress(byte[], int, int, int, int, int)} for description
+   * {@link #decompress(byte[], int, int, int, int, int, int, int)} for
+   * description
    *
    * @param desiredHeight see
-   * {@link #decompress(byte[], int, int, int, int, int)} for description
+   * {@link #decompress(byte[], int, int, int, int, int, int, int)} for
+   * description
    *
    * @param bufferedImageType the image type of the newly-created
    * <code>BufferedImage</code> instance (for instance,
@@ -459,13 +567,14 @@ public class TJDecompressor {
    * decompressed image
    */
   public BufferedImage decompress(int desiredWidth, int desiredHeight,
-    int bufferedImageType, int flags) throws Exception {
-    if(desiredWidth < 0 || desiredHeight < 0 || flags < 0)
+                                  int bufferedImageType, int flags)
+                                  throws Exception {
+    if (desiredWidth < 0 || desiredHeight < 0 || flags < 0)
       throw new Exception("Invalid argument in decompress()");
     int scaledWidth = getScaledWidth(desiredWidth, desiredHeight);
     int scaledHeight = getScaledHeight(desiredWidth, desiredHeight);
     BufferedImage img = new BufferedImage(scaledWidth, scaledHeight,
-      bufferedImageType);
+                                          bufferedImageType);
     decompress(img, flags);
     return img;
   }
@@ -480,9 +589,8 @@ public class TJDecompressor {
   protected void finalize() throws Throwable {
     try {
       close();
-    }
-    catch(Exception e) {}
-    finally {
+    } catch(Exception e) {
+    } finally {
       super.finalize();
     }
   };
@@ -496,15 +604,22 @@ public class TJDecompressor {
 
   private native void decompress(byte[] srcBuf, int size, byte[] dstBuf,
     int desiredWidth, int pitch, int desiredHeight, int pixelFormat, int flags)
-    throws Exception;
+    throws Exception; // deprecated
+
+  private native void decompress(byte[] srcBuf, int size, byte[] dstBuf, int x,
+    int y, int desiredWidth, int pitch, int desiredHeight, int pixelFormat,
+    int flags) throws Exception;
 
   private native void decompress(byte[] srcBuf, int size, int[] dstBuf,
-    int desiredWidth, int pitch, int desiredHeight, int pixelFormat, int flags)
-    throws Exception;
+    int desiredWidth, int stride, int desiredHeight, int pixelFormat,
+    int flags) throws Exception; // deprecated
+
+  private native void decompress(byte[] srcBuf, int size, int[] dstBuf, int x,
+    int y, int desiredWidth, int stride, int desiredHeight, int pixelFormat,
+    int flags) throws Exception;
 
   private native void decompressToYUV(byte[] srcBuf, int size, byte[] dstBuf,
-    int flags)
-    throws Exception;
+    int flags) throws Exception;
 
   static {
     TJLoader.load();
